/* palace-confdialog.c - Palace XMMS configuration dialog
 *
 * $Id: palace-confdialog.c, v1.0 2002/06/07 18:25:16 nitroglycerine Exp $
 * 
 *     XMMS configuration dialog.
 */

#include "palace.h"

GtkWidget *palace_confdialog = NULL;

/* init_config_values() - set widget values to configuration data */
void init_config_values()
{
	GtkWidget *widget = NULL;
	gchar buf[20];
	gint i, old_value = 0;

	ENTER_FUNCTION();
	
	if(!palace_confdialog) {
		debug_printf("Tried to initialize values of a NULL configuration dialog\n");
		LEAVE_FUNCTION();
		return;
	}

	/**** Global Configuration Options ****/
	
	/* Draw graphics */
	widget = find_widget_by_name(palace_confdialog, "draw_graphics_check");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), palacecfg.draw_graphics);

	/* Reverse channel order */
	widget = find_widget_by_name(palace_confdialog, "reverse_channel_order_check");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), palacecfg.reverse_channel_order);

	/* Mode (VU, SPECTRUM, or STEPPER) */
	switch(palacecfg.mode) {
		case VU:
			widget = find_widget_by_name(palace_confdialog, "vu_radio");
			break;
		case SPECTRUM:
			widget = find_widget_by_name(palace_confdialog, "spectrum_radio");
			break;
		case STEPPER:
			widget = find_widget_by_name(palace_confdialog, "stepper_radio");
			break;
	}
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), TRUE);


	/**** Frequency Spectrum Options ****/

	/* Thresholds */
	for(i = 0; i < NUM_BANDS; i++) 
		old_value += palacecfg.threshold[i];

	old_value /= NUM_BANDS;
	widget = find_widget_by_name(palace_confdialog, "all_channels_hscale");
	gtk_object_set_data(GTK_OBJECT(GTK_RANGE(widget)->adjustment), "old_value", (gpointer)old_value);
	debug_printf("I just set old_value on widget 0x%08lx to %i.\n", (unsigned long)GTK_OBJECT(GTK_RANGE(widget)->adjustment), old_value);
	gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, old_value);

	for(i = 0; i < NUM_BANDS; i++) {
		sprintf(buf, "channel_%i_hscale", i + 1);
		widget = find_widget_by_name(palace_confdialog, buf);
		gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, palacecfg.threshold[i]);
		/* gtk_adjustment_value_changed(GTK_RANGE(widget)->adjustment); */
	}

	/* Variable size frequency bars */
	widget = find_widget_by_name(palace_confdialog, "variable_frequency_bars_check");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), palacecfg.variable_frequency_bars);

	/* Frequency bar falloff rate */
	widget = find_widget_by_name(palace_confdialog, "frequency_falloff_hscale");
	gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, palacecfg.frequency_falloff_rate);

	
	/**** Vu-meter Options ****/

	/* VU power scale */
	widget = find_widget_by_name(palace_confdialog, "vu_power_hscale");
	gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, palacecfg.vu_power_scale);


	/**** Stepper Motor Options ****/

	/* Number of stepper pins */
	/*
	widget = find_widget_by_name(palace_confdialog, "stepper_pins");
	gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, palacecfg.stepper_pins);
	*/

	/* Stepper strobe rate */
	widget = find_widget_by_name(palace_confdialog, "stepper_speed_hscale");
	gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, 10 - palacecfg.stepper_strobe_rate);

	/* Vary stepper speed */
	widget = find_widget_by_name(palace_confdialog, "stepper_vary_speed_check");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), palacecfg.stepper_vary_speed);


	/**** Parallel Port Output Options ****/
	
	/* Send data to parallel port */
	widget = find_widget_by_name(palace_confdialog, "send_lpt_data_check");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), palacecfg.send_lpt_data);

	/* Base LPT address */
	widget = find_widget_by_name(palace_confdialog, "lpt_address");
	sprintf(buf, "0x%03x", palacecfg.lpt_address);
	gtk_entry_set_text(GTK_ENTRY(widget), buf);


	/**** Beat Detection Options ****/
	
	/* Flash on beats */
	widget = find_widget_by_name(palace_confdialog, "flash_on_beats_check");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), palacecfg.flash_on_beats);

	/* BPM to look for beats */
	widget = find_widget_by_name(palace_confdialog, "target_bpm_hscale");
	gtk_adjustment_set_value(GTK_RANGE(widget)->adjustment, palacecfg.target_bpm);

	LEAVE_FUNCTION();
}

