/* palace-config.h - Palace configuration
 *
 * $Id: palace-config.h, v1.0 2002/06/07 18:25:16 nitroglycerine Exp $
 * 
 */

#ifndef _PALACE_CONFIG_H_
#define _PALACE_CONFIG_H_

/* Defines */
#define XMMS_CFG_SECTION	"Palace: Parallel Lightshow"
#define DEFAULTLPTPORT		0x378
#define DEFAULTSENSE		75
#define DEFAULTLOCK		25
#define MAX_SENSE		100


/* Types */

/* Palace configuration structure */
typedef struct {
	/**** Global Configuration Options ****/
	
	/* Draw on-screen graphics (bool) */
	gboolean draw_graphics;
	
	/* Reverse channel order (bool) */
	gboolean reverse_channel_order;

	/* Mode (VU, SPECTRUM, or STEPPER) (enum) */
	enum { VU, SPECTRUM, STEPPER } mode;
	
	
	/**** Frequency Spectrum Options ****/

	/* Channel thresholds (level at which light is activated) (unsigned int array) */
	guint threshold[NUM_BANDS];
	
	/* Draw variable size frequency bars (bool) */
	gboolean variable_frequency_bars;

	/* Falloff speed (pixels per frame) (int) */
	gint frequency_falloff_rate;


	/**** Vu-meter Options ****/

	/* VU power scale (float) */
	gfloat vu_power_scale;


	/**** Stepper Motor Options ****/

	/* Number of stepper pins (2-8) (unsigned char) */
	guint stepper_pins;

	/* Stepper strobe rate (Hz - negative values reverse direction) (int) */
	/* Revolutions per second is the strobe rate / number of stepper pins */
	/* TODO: implement actual Hz counter, rather than frame counter */
	gint stepper_strobe_rate;

	/* Vary stepper speed based on music */
	gboolean stepper_vary_speed;
	
	
	/**** Parallel Port Output Options ****/

	/* Send data to the parallel port (bool) */
	gboolean send_lpt_data;
	
	/* Parallel port I/O address (unsigned int) */
	guint lpt_address;
	
	
	/**** Beat Detection Options ****/

	/* Flash on beats (bool) */
	gboolean flash_on_beats;

	/* BPM to look for beats (unsigned int) */
	guint target_bpm;

} palace_config_t;


/* Functions from palace-config.c */
void palace_read_config();
void palace_write_config();


/* Variables */
extern palace_config_t palacecfg;


#endif /* _PALACE_CONFIG_H_ */

