/* palace-gtk-util.c - Functions to help with Gtk+ stuff
 *
 * $Id: palace-gtk-util.c, v1.0 2002/06/13 20:10:45 nitroglycerine Exp $
 */

#include "palace.h"


/* Credit due to Deborah Swayne <dfs research att com> for posting the next
 * two functions to gtk-list in August 2001.
 * 
 * The functions have been modified to suit the needs of Palace.
 */

/*--------------------------------------------------------------------*/
/*--- Find a widget by name, starting from an enclosing container ----*/
/*--------------------------------------------------------------------*/

static gboolean widget_name_p (GtkWidget *w, gchar *name)
{
	ENTER_FUNCTION();
	
	if (strcmp (gtk_widget_get_name (w), name) == 0) {
		return TRUE;
	}
	else return FALSE;

	LEAVE_FUNCTION();
}

/*
 * parent must be a container: loop through its children, finding 
 * a widget with the name 'name'
 */
GtkWidget *blind_widget_by_name (GtkWidget *parent, gchar *name)
{
	GtkWidget *w, *namedw = NULL;
	GList *children, *l;

	ENTER_FUNCTION();
	
	if (widget_name_p (parent, name))
		namedw = parent;
	else {
		if (GTK_CONTAINER(parent)) {
			children = gtk_container_children (GTK_CONTAINER(parent));
			for (l = children; l; l = l->next) {
				if (GTK_IS_WIDGET(l->data)) {
					w = GTK_WIDGET (l->data);
					if (widget_name_p (w, name)) {
						namedw = w;
						break;
					}
					if (GTK_IS_CONTAINER (w)) {
						namedw = find_widget_by_name (w, name);
						if (namedw != NULL)
							break;
					}
				}
			}
		}
	}

	LEAVE_FUNCTION();
	
	return namedw;
}

GtkWidget* find_widget_by_name (GtkWidget *widget, const gchar *widget_name)
{
	GtkWidget *parent, *found_widget;
	
	for (;;) {
		if (GTK_IS_MENU (widget))
			parent = gtk_menu_get_attach_widget (GTK_MENU (widget));
		else
			parent = widget->parent;
		if (parent == NULL)
			break;
		widget = parent;
	}
	
	found_widget = (GtkWidget*) gtk_object_get_data (GTK_OBJECT (widget), widget_name);
	
	if (!found_widget)
		g_warning ("Widget not found: %s", widget_name);
	return found_widget;
}


