/* palace.c - main Palace DCI source file
 *
 * $Id: palace.c, v1.0 2002/06/08 16:10:45 nitroglycerine Exp $
 *
 *     Main Palace file - initialization and communication between palace
 * components is handled here.
 */


#if HAVE_SYS_IO_H
#include <sys/io.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#include <sys/time.h>
#include <glib.h>
#include <gtk/gtk.h>
#include <pthread.h>
#include <limits.h>

#include "palace.h"

#include "renderwindow/interface.h"
#include "confdialog/interface.h"
#include "confdialog/support.h"

#define MAX_BIGVU	4500000
#define MIX		0

GtkWidget *palace_renderwindow = (GtkWidget *)NULL;
static GtkWidget *palace_area = (GtkWidget *)NULL;
static GtkPixmap *bg_pixmap = NULL;
static GtkPixmap *palace_logo_dark = NULL;
static GtkPixmap *palace_logo_light = NULL;
static GdkPixmap *draw_pixmap = (GdkPixmap *)NULL;
static GtkPixmap *green_bar = NULL;
static GtkPixmap *yellow_bar = NULL;
static GtkPixmap *red_bar = NULL;
static GdkGC *gc = (GdkGC *)NULL;
static gint16 bar_heights[NUM_BANDS];
static gdouble scale;
static gboolean playing = FALSE;
/* stepper stuff */
static gint curstep = 0;
static gint pstep = 0; /* counter so the stepper doesn't go too fast */

/* Continuous VU stuff */
static gfloat vu_level = 0;
static gfloat max_vu_level = INT_MIN;
static gfloat delta_vu_level = 0;
static gfloat max_delta_vu_level = INT_MIN;
static gfloat min_delta_vu_level = INT_MAX;
static gfloat last_vu_level = 0;
static gint bigvu_overflow = 0;
static gint last_bigvu = -1;
static gint bigvu = 0;
static gint delta_bigvu = 0;
static gint max_bigvu = INT_MIN;
static gint max_delta_bigvu = INT_MIN;
static gint min_delta_bigvu = INT_MAX;
static gint last_delta_bigvu = 0;

/* About Box */
static GtkWidget *about_box = (GtkWidget *)NULL;

static void draw_func(gint h[NUM_BANDS]);

static void palace_init(void);
static void palace_cleanup(void);
static void palace_about(void);
static void palace_configure(void);
void palace_playback_start_internal(void);
void palace_playback_stop_internal(void);
void palace_playback_start(void);
void palace_playback_stop(void);
static void palace_render_freq(gint16 data[2][256]);

palace_state_t palace_state = {0, 0, 0, {0, 0}, {0, 0}, NULL};

VisPlugin palace_plugin_info = {
	NULL,			/* (handle) */
	NULL,			/* (filename) */
	0,			/* (xmms_session) */
	"Palace - Parallel Port Light Show",	/* desc */
	0,			/* pcm_chs */
	1,			/* freq_chs */
	palace_init,		/* init */
	palace_cleanup,		/* cleanup */
	palace_about,		/* about */
	palace_configure,	/* configure */
	NULL,			/* (disable_plugin) */
	palace_playback_start,	/* playback_start */
	palace_playback_stop,	/* playback_stop */
	NULL,			/* render_pcm */
	palace_render_freq	/* render_freq */
};

VisPlugin *get_vplugin_info(void)
{
	ENTER_FUNCTION();

	add_pixmap_directory (PACKAGE_DATA_DIR "/pixmaps");
	/* Uncommenting the next line is a small security risk */
	/* add_pixmap_directory (PACKAGE_SOURCE_DIR "/pixmaps"); */

	LEAVE_FUNCTION();
	return &palace_plugin_info;
}


void palace_reset_beat_detection()
{
	palace_state.n_flashes = 0;
	palace_state.manual_flash = 0;
	palace_state.mean_time_between_flashes = 60.0 / palacecfg.target_bpm;
	palace_state.thistime.tv_sec = 0;
	palace_state.thistime.tv_usec = 0;
	palace_state.lasttime.tv_sec = 0;
	palace_state.lasttime.tv_usec = 0;
}


static void palace_init()
{
	GdkColor color;
	
	ENTER_FUNCTION();
	
	if (palace_renderwindow != NULL) {
		MADE_IT_HERE();
		LEAVE_FUNCTION();
		return;
	}
	
	if(!palace_confdialog)
		palace_read_config();
	
	debug_printf("palace_renderwindow before assignment is: 0x%08lx\n", (unsigned long)palace_renderwindow);
	palace_renderwindow = create_palace_render_window();
	debug_printf("palace_renderwindow after assignment is: 0x%08lx\n", (unsigned long)palace_renderwindow);

	gtk_widget_show(palace_renderwindow);

	gtk_window_set_title(GTK_WINDOW(palace_renderwindow), "Palace - Parallel Port Light Show");
	gtk_widget_realize(palace_renderwindow);
	palace_logo_dark = GTK_PIXMAP(create_pixmap(palace_renderwindow, "palace.xpm"));
	palace_logo_light = GTK_PIXMAP(create_pixmap(palace_renderwindow, "palace_light.xpm"));
	bg_pixmap = palace_logo_dark;

	debug_printf("about to set back pixmap of window 0x%08lx to bg_pixmap->pixmap (0x%08lx) at line %i of file %s.\n", 
		     (unsigned long)palace_renderwindow, (unsigned long)bg_pixmap->pixmap, __LINE__, __FILE__);

	gdk_window_set_back_pixmap(palace_renderwindow->window, bg_pixmap->pixmap, 0);
	
	gc = gdk_gc_new(palace_renderwindow->window);
	
	draw_pixmap = gdk_pixmap_new(palace_renderwindow->window, WIDTH, HEIGHT, gdk_visual_get_best_depth());
	gdk_draw_rectangle(draw_pixmap, gc, TRUE, 0, 0, WIDTH, HEIGHT);
	
	MADE_IT_HERE();
	
	green_bar = GTK_PIXMAP(create_pixmap(palace_renderwindow, "green_bar.xpm"));
	yellow_bar = GTK_PIXMAP(create_pixmap(palace_renderwindow, "yellow_bar.xpm"));
	red_bar = GTK_PIXMAP(create_pixmap(palace_renderwindow, "red_bar.xpm"));

	MADE_IT_HERE();
	
	if(palacecfg.send_lpt_data) {
		send_ok = palace_control_init() ? FALSE : TRUE;
		if (!send_ok) {
			fprintf(stderr, "Palace was unable to initialize parallel port output\n");
			gtk_window_set_title(GTK_WINDOW(palace_renderwindow), "Palace -- Port Inactive");
		} else {
			gtk_window_set_title(GTK_WINDOW(palace_renderwindow), "Palace -- Port Active");
		}
	}

	MADE_IT_HERE();

	color.red = 0xFFFF;
	color.green = 0xFFFF;
	color.blue = 0xFFFF;
	scale = HEIGHT / log10(256);
	gdk_color_black(gdk_colormap_get_system(), &color);
	gdk_gc_set_foreground(gc, &color);

	palace_area = GTK_WIDGET(find_widget_by_name(palace_renderwindow, "palace_drawing_area"));
	
	gdk_window_set_back_pixmap(palace_area->window, bg_pixmap->pixmap, 0);
	gdk_window_clear(palace_renderwindow->window);
	gdk_window_clear(palace_area->window);

	palace_reset_beat_detection();
	
	LEAVE_FUNCTION();
}


static void palace_cleanup(void)
{
	ENTER_FUNCTION();
	
	if(playing)
		palace_playback_stop();

	if(gc != (GdkGC *)NULL) {
		gdk_gc_unref(gc);
		gc = (GdkGC *)NULL;
	}
	if(palace_renderwindow != (GtkWidget *)NULL) {
		gtk_widget_destroy(palace_renderwindow);
		palace_renderwindow = (GtkWidget *)NULL;
	}
	if(palace_confdialog) {
		gtk_widget_destroy(palace_confdialog);
		palace_confdialog = NULL;
	}

	if(send_ok) {
		palace_control_cleanup();
	}

	if(palace_state.title) {
		free(palace_state.title);
		palace_state.title = NULL;
	}
		
	debug_printf("VU level is: %f.  Max VU is: %f.\n", vu_level, max_vu_level);
	debug_printf("Delta VU is: %f.  Max Delta VU is: %f.  Min Delta VU is: %f.\n", delta_vu_level, max_delta_vu_level, min_delta_vu_level);
	debug_printf("Big vu level is: %i.  Delta big vu level is: %i.\n", bigvu, delta_bigvu);
	debug_printf("Max big vu level is: %i.  Max delta bigvu is: %i.  Min delta bigvu is: %i.\n", max_bigvu, max_delta_bigvu, min_delta_bigvu);

	LEAVE_FUNCTION();
}

static void draw_func(gint h[NUM_BANDS])
{
	gint i/* , lptdata = 0 */;
	gint k = 0;
	/* gint16 heights[NUM_BANDS]; */
	GdkColor color;
	double time_since_last_flash = 60.0 / palacecfg.target_bpm;
	double seconds_per_beat_low = 60.0 / (palacecfg.target_bpm + 5.0);
	double seconds_per_beat_high = 60.0 / (palacecfg.target_bpm - 5.0);
	gint v;

	if(palace_state.n_flashes == 0) {
		gettimeofday(&palace_state.thistime, NULL);
		gettimeofday(&palace_state.lasttime, NULL);
	}

	if(!palace_state.title || 
	   strcmp(xmms_remote_get_playlist_file(palace_plugin_info.xmms_session, xmms_remote_get_playlist_pos(palace_plugin_info.xmms_session)), 
		  palace_state.title)) {
		if(palace_state.title) {
			free(palace_state.title);
		}
		palace_state.title = strdup(xmms_remote_get_playlist_file(palace_plugin_info.xmms_session, 
					    xmms_remote_get_playlist_pos(palace_plugin_info.xmms_session)));
		debug_printf("New song: %s\n", palace_state.title);
		/* switch variables for new song */
		palace_reset_beat_detection();
	}

	if(palacecfg.draw_graphics) {
		/* Draw background first */
		gdk_draw_rectangle(draw_pixmap, gc, TRUE, 0, 0, WIDTH, HEIGHT);
		gdk_draw_pixmap(draw_pixmap, gc, bg_pixmap->pixmap, 0, 0, 0, 0, WIDTH, HEIGHT);
		
		/* Draw smooth line if in variable frequency bars mode */
		if(palacecfg.variable_frequency_bars && palacecfg.mode == SPECTRUM) {
			color.red = color.green = color.blue = 0x4444;
			gdk_color_alloc(gdk_colormap_get_system(), &color);
			gdk_gc_set_foreground(gc, &color);
			gdk_draw_line(draw_pixmap, gc, 0, HEIGHT / 2 - 1, WIDTH, HEIGHT / 2 - 1);
			gdk_draw_line(draw_pixmap, gc, 0, HEIGHT / 2 + 1, WIDTH, HEIGHT / 2 + 1);
			color.red = color.green = color.blue = 0xAAAA;
			gdk_color_alloc(gdk_colormap_get_system(), &color);
			gdk_gc_set_foreground(gc, &color);
			gdk_draw_line(draw_pixmap, gc, 0, HEIGHT / 2, WIDTH, HEIGHT / 2);
			color.red = color.green = color.blue = 0x0;
			gdk_color_alloc(gdk_colormap_get_system(), &color);
			gdk_gc_set_foreground(gc, &color);
		}

		/* Draw the bars */
		/* Putting the if outside the for increases code size but it is faster */
		if(!palacecfg.reverse_channel_order) {
			for(i = 0; i < NUM_BANDS; i++) {
				v = h[i];
				if (i < 4) {
					gdk_draw_pixmap(draw_pixmap, gc, green_bar->pixmap,
							0, HEIGHT - 1 - v,
							i * (WIDTH / NUM_BANDS + 2), HEIGHT - 1 - v,
							(WIDTH / NUM_BANDS) - 1, v);
				} else if (i < 6) {
					gdk_draw_pixmap(draw_pixmap, gc, yellow_bar->pixmap,
							0, HEIGHT - 1 - v,
							i * (WIDTH / NUM_BANDS + 2), HEIGHT - 1 - v,
							(WIDTH / NUM_BANDS) - 1, v);
				} else {
					gdk_draw_pixmap(draw_pixmap, gc, red_bar->pixmap,
							0, HEIGHT - 1 - v,
							i * (WIDTH / NUM_BANDS + 2), HEIGHT - 1 - v,
							(WIDTH / NUM_BANDS) - 1, v);
				}
			}
		} else {
			for(i = 0; i < NUM_BANDS; i++) {
				v = h[NUM_BANDS - i - 1];
				if (i < 2) {
					gdk_draw_pixmap(draw_pixmap, gc, red_bar->pixmap,
							0, HEIGHT - 1 - v,
							i * (WIDTH / NUM_BANDS + 2), HEIGHT - 1 - v,
							(WIDTH / NUM_BANDS) - 1, v);
				} else if (i < 4) {
					gdk_draw_pixmap(draw_pixmap, gc, yellow_bar->pixmap,
							0, HEIGHT - 1 - v,
							i * (WIDTH / NUM_BANDS + 2), HEIGHT - 1 - v,
							(WIDTH / NUM_BANDS) - 1, v);
				} else {
					gdk_draw_pixmap(draw_pixmap, gc, green_bar->pixmap,
							0, HEIGHT - 1 - v,
							i * (WIDTH / NUM_BANDS + 2), HEIGHT - 1 - v,
							(WIDTH / NUM_BANDS) - 1, v);
				}
			}
		}

		/* Draw the VU level lines */
#ifdef DEBUG
		color.red = 0x4444;
		color.green = 0xFFFF;
		color.blue = 0x7777;
		gdk_color_alloc(gdk_colormap_get_system(), &color);
		gdk_gc_set_foreground(gc, &color);
		i = (bigvu * WIDTH) / MAX_BIGVU;
		gdk_draw_line(draw_pixmap, gc, i, 0, i, HEIGHT);
		gdk_draw_line(draw_pixmap, gc, i + 1, 0, i + 1, HEIGHT);
		gdk_draw_line(draw_pixmap, gc, i - 1, 0, i - 1, HEIGHT);
		color.red = 0x3333;
		color.green = 0x5555;
		color.blue = 0xFFFF;
		gdk_color_alloc(gdk_colormap_get_system(), &color);
		gdk_gc_set_foreground(gc, &color);
		i = (delta_bigvu * WIDTH) / (MAX_BIGVU * 2) + (WIDTH / 2);
		gdk_draw_line(draw_pixmap, gc, i, 0, i, HEIGHT);
		gdk_draw_line(draw_pixmap, gc, i + 1, 0, i + 1, HEIGHT);
		gdk_draw_line(draw_pixmap, gc, i - 1, 0, i - 1, HEIGHT);
#endif /* DEBUG	*/

		/* flash on beat */
		/* calculate elapsed time in seconds since last flash */
		gettimeofday(&palace_state.thistime, NULL);
		time_since_last_flash = ((double)palace_state.thistime.tv_usec / 1000000.0 + (double)palace_state.thistime.tv_sec) - 
					((double)palace_state.lasttime.tv_usec / 1000000.0 + (double)palace_state.lasttime.tv_sec);
		if(time_since_last_flash >= palace_state.mean_time_between_flashes && palace_state.n_flashes >= 25)
			palace_state.manual_flash++;
		if(palace_state.manual_flash >= 8)
			palace_state.manual_flash = -4;
		
		if((bigvu > 7500 && delta_bigvu >= 1500 /* (int)((float)bigvu * 0.01 * (50.0 - (float)bigvu/149852.0)) */ && 
		    last_delta_bigvu < -100 && palacecfg.flash_on_beats && 
		   (palace_state.n_flashes < 10 || time_since_last_flash >= 0.75 * palace_state.mean_time_between_flashes))) {
			palace_state.lasttime = palace_state.thistime;

			palace_state.n_flashes++;
			/* Limit range to user configuration option +/- 5bpm */
			if(time_since_last_flash >= seconds_per_beat_low && 
				time_since_last_flash <= seconds_per_beat_high && 
				palace_state.manual_flash < 0) {
			/* update average number of flashes */
				palace_state.mean_time_between_flashes = (double)time_since_last_flash + 
							    (double)(palace_state.n_flashes - 1) * palace_state.mean_time_between_flashes;
				palace_state.mean_time_between_flashes /= palace_state.n_flashes;
			}

			/* set color to white and flash lights */
			color.red = color.green = color.blue = 0xFFFF;
			bg_pixmap = palace_logo_light;
			// lptdata = 0xFF;

			curstep++;
			if(curstep >= palacecfg.stepper_pins)
				curstep = 0;
			
			/* display debug information about the flashes */
			debug_printf("**FLASH**: n=%i, t=%fs, tm=%fs\n", 
				     palace_state.n_flashes,
				     time_since_last_flash, 
				     palace_state.mean_time_between_flashes);
		} else {
			/* lptdata = 0x0; */
			color.red = color.green = color.blue = 0x0;
			bg_pixmap = palace_logo_dark;
		}

		gdk_color_alloc(gdk_colormap_get_system(), &color);
		gdk_gc_set_foreground(gc, &color);
		GDK_THREADS_ENTER();
		gdk_window_clear(palace_area->window);
		GDK_THREADS_LEAVE();
	}
}


void palace_playback_start_internal(void)
{
	ENTER_FUNCTION();
	
	if (!palacecfg.draw_graphics)
		return;
	if (!playing)
		return;
	if (palace_renderwindow != (GtkWidget *)NULL) {
		gdk_window_set_back_pixmap(palace_area->window, draw_pixmap, 0); 
		/* gdk_window_set_back_pixmap(palace_area->window, bg_pixmap->pixmap, 0); */
		gdk_window_clear(palace_area->window);
	}

	LEAVE_FUNCTION();
}


static void pause_graphics()
{
	int i = 0, lptdata = 0;

	ENTER_FUNCTION();

	debug_printf("BAD - entered a function that shouldn't be entered\n");
	
	while (!playing) {
		for(i = 0; i < 8; i++) {
			lptdata &= 1 << i;
			if(palacecfg.send_lpt_data)
				palace_control_output(lptdata);
			/* usleep(50000); */
		}
		for(i = 0; i < 8; i++) {
			lptdata &= !(1 << i);
			if(palacecfg.send_lpt_data)
				palace_control_output(lptdata);
			/* usleep(50000); */
		}
		for(i = 7; i >= 0; i--) {
			lptdata &= 1 << i;
			if(palacecfg.send_lpt_data)
				palace_control_output(lptdata);
			/* usleep(50000); */
		}
		for (i = 7; i >= 0; i--) {
			lptdata &= !(1 << i);
			if(palacecfg.send_lpt_data)
				palace_control_output(lptdata);
			/* usleep(50000); */
		}
	}
	if(palacecfg.send_lpt_data)
		palace_control_output(0);

	LEAVE_FUNCTION();
}


void palace_playback_stop_internal(void)
{
	ENTER_FUNCTION();
	
	if (!playing)
		return;
	
	if (GTK_WIDGET_REALIZED(palace_area)) {
		gdk_window_set_back_pixmap(palace_area->window, bg_pixmap->pixmap, 0);
		gdk_window_clear(palace_area->window);
	}
	
	LEAVE_FUNCTION();
}


void palace_playback_start(void)
{
	ENTER_FUNCTION();
	
	if (playing)
		return;
	playing = TRUE;
	
	palace_playback_start_internal();
	
	LEAVE_FUNCTION();
}


void palace_playback_stop(void)
{
	ENTER_FUNCTION();
	
	if (!playing)
		return;
	palace_playback_stop_internal();

	if (send_ok && palacecfg.send_lpt_data)
		palace_control_output(0);

	playing = FALSE;
	/*
	if(pthread_cancel(draw_thread)) {
		perror("Error canceling thread");
	}
	if(pthread_join(draw_thread, NULL)) {
		perror("Error waiting for and joining thread");
	}
	*/

	LEAVE_FUNCTION();
}

void palace_render_freq(gint16 data[2][256])
{
	gint i, c, y;
	gint h[NUM_BANDS];
	gfloat v;
	int lptdata = 0;

	/* Logarithmic frequency span (each input band band is samplerate/512 Hz wide) */
	gint xscale[] =
	{
		0, 2, 5, 10, 20, 40, 74, 137, 255
	};
	
	/* Exponential frequency span (doesn't work well)
	gint xscale[] = {
		0, 1, 2, 4, 8, 16, 32, 64, 128, 255
	};
	*/

	memset(h, 0, sizeof(h));

	/* Check for a non-existing window (this would happen if XMMS ever 
	 * called render_freq before init, which isn't really possible, but 
	 * it never hurts to check... 
	 */
	if(palace_renderwindow == (GtkWidget *)NULL)
		return;

	/* Calculate VU level */
	last_vu_level = vu_level;
	last_bigvu = bigvu;
	vu_level = 0;
	bigvu = 0;
	for (i = 0; i < 256; i++) {
#if MIX
		gfloat d = (gfloat)(data[0][i] + data[1][i]) / 2.0;
#else
		gfloat d = (gfloat)data[0][i];
#endif
		/* 
		 * the (2.0 - i^2 / 43690.66666667) causes the lowest band
		 * to be mutiplied by 2.0, and the highest band to be multiplied
		 * by 0.5.
		 */
		gint v;
		
		d /= 256;
		d = d * d;

		v = (int)(d * (2.0 - (float)i * (float)i / 43690.66666667));

		/* average: */
		/*
		bigvu += v;
		*/
		
		/* max: */
		/*
		if(bigvu < (v * 256))
			bigvu = v * 256;
		*/

		/* bass: */
		if(i < 3 && bigvu < (v * 256))
			bigvu = v * 256;
	}

	vu_level = bigvu / 256;
	delta_vu_level = vu_level - last_vu_level;

#define VU_FALLOFF
#ifdef VU_FALLOFF
	/* XXX Maybe change it so that the attack/decay rate is based on how
	 * large the difference between the current and target values is? 
	 */
	bigvu += bigvu_overflow;
	if(bigvu < last_bigvu) {
		/* Limit vu-level falloff rate, to make it look more smooth */
		if(bigvu < last_bigvu - (8192 * palacecfg.frequency_falloff_rate)) {
			bigvu = last_bigvu - (8192 * palacecfg.frequency_falloff_rate);
		}
	} else {
		/* Likewise, except for attack rate */
		if(bigvu > last_bigvu + (131072 * palacecfg.frequency_falloff_rate)) {
			bigvu_overflow = bigvu - (last_bigvu + (131072 * palacecfg.frequency_falloff_rate));
			bigvu = last_bigvu + (131072 * palacecfg.frequency_falloff_rate);
		}
	}
#endif

	/* Loudness change since last frame and frame before -- used by beat detection */
	last_delta_bigvu = delta_bigvu;
	delta_bigvu = bigvu - last_bigvu;

	/* Calculate extrema to display at plugin exit */
	if(bigvu > max_bigvu) {
		max_bigvu = bigvu;
		max_vu_level = vu_level;
	}
	if(delta_bigvu > max_delta_bigvu) {
		max_delta_bigvu = delta_bigvu;
		max_delta_vu_level = delta_vu_level;
	}
	if(delta_bigvu < min_delta_bigvu) {
		min_delta_bigvu = delta_bigvu;
		min_delta_vu_level = delta_vu_level;
	}
	
	/* Calculate frequency bands */
	for(i = 0; i < NUM_BANDS; i++) {
		for(c = xscale[i], y = 0; c < xscale[i + 1]; c++) {
#if MIX
			gint d = (data[0][c] + data[1][c]) / 2;
#else
			gint d = data[0][c];
#endif

			if(d > y)
				y = d;
		}
		y /= 128; /* Scale range from (-32768, 32767) to (-256, 255) */
		if(y > 0) {
			y = (gint)(log10(y) * scale);
			if(y > HEIGHT - 1)
				y = HEIGHT - 1;
		} else {
			y = 0;
		}
		bar_heights[i] -= palacecfg.frequency_falloff_rate;
		if(y > bar_heights[i])
			bar_heights[i] = y;
	}

	switch(palacecfg.mode) {
		case SPECTRUM:
			for(i = 0; i < NUM_BANDS; i++) {
				// Is this band above the set threshold?
				if(bar_heights[i] > palacecfg.threshold[i]) {
					h[i] = HEIGHT;
				} else {
					h[i] = 0;
				}
				
				if(palacecfg.variable_frequency_bars) {
					int k = 0;
					// Maximum height is twice threshold (so that threshold is always the middle of the window) 
					h[i] = palacecfg.threshold[i] * 2;
					// Scale and clamp the value for this band from 0..whatever to 0..threshold[i]*2
					k = bar_heights[i];
					if(k > h[i])
						k = h[i];
					h[i] = (k * HEIGHT) / h[i];
				}
			}
			break;

		case VU:
			/* V should now be from 0..1 */
			v = vu_level / 19850.0;

			v = pow(v, palacecfg.vu_power_scale);

			v *= 8.0;
			
			for(i = 0; i < NUM_BANDS; i++) {
				if(i < v) {
					h[i] = HEIGHT;
				} else {
					h[i] = 0;
				}
			}
			break;

		case STEPPER:
			h[curstep] = HEIGHT;
			/* 
			 * Don't do normal stepper motor stuff when flash on
			 * beats is enabled, since flash on beats will increment
			 * the stepper motor each beat.
			 */
			if(!palacecfg.flash_on_beats) {
				if(pstep == 0) {
					curstep++;
					if(curstep >= palacecfg.stepper_pins)
						curstep = 0;
				}
				pstep++;
				if(pstep >= palacecfg.stepper_strobe_rate) {
					pstep = 0;
				}
			}
			
			/* Set the stepper speed based on music loudness */
			if(palacecfg.stepper_vary_speed) {
				palacecfg.stepper_strobe_rate = (int)(20.0 - vu_level / 30.0);
				if (palacecfg.stepper_strobe_rate < 0)
					palacecfg.stepper_strobe_rate = 0;
			}
			break;

		default:
			debug_printf("Weird..  Mode is an unknown mode!  Setting to SPECTRUM...\n");
			palacecfg.mode = SPECTRUM;
			break;
	}

	/* Calculate port output */
	for(i = 0; i < NUM_BANDS; i++) {
		if(h[palacecfg.reverse_channel_order ? (NUM_BANDS - 1 - i) : i] > (HEIGHT / 2)) {
			lptdata |= 1 << i;
		}
	}

	/* Output to the port */
	if(send_ok && palacecfg.send_lpt_data) {
		palace_control_output(lptdata);
	}
	
	draw_func(h);

	// LEAVE_FUNCTION();
	
	return;
}


static void palaceabout_close_cb(GtkWidget *w, gpointer data)
{
	ENTER_FUNCTION();
	
	gtk_widget_destroy(about_box);
	about_box = (GtkWidget *)NULL;

	LEAVE_FUNCTION();
}


static void palace_about(void)
{
	GtkWidget *button, *label;

	ENTER_FUNCTION();
	
	if (about_box != (GtkWidget *)NULL)
		return;
	about_box = gtk_dialog_new();
	gtk_window_set_title(GTK_WINDOW(about_box),
		"About Palace");
	gtk_window_set_position(GTK_WINDOW(about_box),
		GTK_WIN_POS_MOUSE);
	gtk_container_border_width(GTK_CONTAINER(about_box), 5);
	label = gtk_label_new("Palace version " VERSION "\n\
Palace Parallel Port Light Show\n\n\
By Mike Bourgeous <nitrogen@slimetech.com>\n\n\
Palace was originally based on XPLSISNJASP by\n\
Vlatko Kosturjak, Robert Avilov, Steve Pomeroy.\n\n\
Palace is available from:\n\
http://palace-dci.sourceforge.net/\n\
XPLSISNJASP is available from:\n\
http://www.na.linux.hr/projects/xplsisnjasp/\n\
\n\
");
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about_box)->vbox),
		label, TRUE, TRUE, 0);
	button = gtk_button_new_with_label(" Close ");
	gtk_signal_connect(GTK_OBJECT(button), "clicked",
		GTK_SIGNAL_FUNC(palaceabout_close_cb), NULL);
	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(about_box)->action_area),
		button, FALSE, FALSE, 0);
	gtk_widget_show(label);
	gtk_widget_show(button);
	gtk_widget_show(about_box);
	gtk_widget_grab_focus(button);

	LEAVE_FUNCTION();
}


void palace_configure()
{
	ENTER_FUNCTION();
	
	if (palace_confdialog != NULL) {
		debug_printf("It seems a configuration dialog is already present.\n");
		LEAVE_FUNCTION();
		return;
	}

	palace_confdialog = create_palace_confdialog();
	if(!palace_renderwindow)
		palace_read_config();
	else
		init_config_values();
	gtk_widget_show(palace_confdialog);

	LEAVE_FUNCTION();
}


